@@ -399,8 +399,7 @@ unless ($define{'USE_ITHREADS'}) {
 			 );
 }
 
-unless (   $define{'USE_ITHREADS'}
-        && $define{'HAS_NEWLOCALE'})
+unless ($define{'HAS_NEWLOCALE'} && ! $define{'NO_POSIX_2008_LOCALE'})
 {
     ++$skip{$_} foreach qw(
         PL_C_locale_obj
@@ -482,6 +481,10 @@ unless ($define{'MULTIPLICITY'}) {
     ++$skip{$_} foreach qw(
 		    PL_interp_size
 		    PL_interp_size_5_18_0
+                    PL_sv_yes
+                    PL_sv_undef
+                    PL_sv_no
+                    PL_sv_zero
 			 );
 }
 
