@@ -147,7 +147,7 @@
     } STMT_END
 
 #define THREAD_CREATE(thr, f)	(thr->self = cthread_fork(f, thr), 0)
-#define THREAD_POST_CREATE(thr)
+#define THREAD_POST_CREATE(thr)	NOOP
 
 #define THREAD_RET_TYPE		any_t
 #define THREAD_RET_CAST(x)	((any_t) x)
@@ -208,10 +208,18 @@
     } STMT_END
 #  endif
 
+#  ifdef PERL_TSA_ACTIVE
+#    define perl_pthread_mutex_lock(m) perl_tsa_mutex_lock(m)
+#    define perl_pthread_mutex_unlock(m) perl_tsa_mutex_unlock(m)
+#  else
+#    define perl_pthread_mutex_lock(m) pthread_mutex_lock(m)
+#    define perl_pthread_mutex_unlock(m) pthread_mutex_unlock(m)
+#  endif
+
 #  define MUTEX_LOCK(m) \
     STMT_START {						\
 	int _eC_;						\
-	if ((_eC_ = pthread_mutex_lock((m))))			\
+	if ((_eC_ = perl_pthread_mutex_lock((m))))			\
 	    Perl_croak_nocontext("panic: MUTEX_LOCK (%d) [%s:%d]",	\
 				 _eC_, __FILE__, __LINE__);	\
     } STMT_END
@@ -219,7 +227,7 @@
 #  define MUTEX_UNLOCK(m) \
     STMT_START {						\
 	int _eC_;						\
-	if ((_eC_ = pthread_mutex_unlock((m))))			\
+	if ((_eC_ = perl_pthread_mutex_unlock((m))))			\
 	    Perl_croak_nocontext("panic: MUTEX_UNLOCK (%d) [%s:%d]",	\
 				 _eC_, __FILE__, __LINE__);	\
     } STMT_END
@@ -336,7 +344,7 @@
 #  define ALLOC_THREAD_KEY \
     STMT_START {						\
 	if (pthread_key_create(&PL_thr_key, 0)) {		\
-            write(2, STR_WITH_LEN("panic: pthread_key_create failed\n")); \
+            PERL_UNUSED_RESULT(write(2, STR_WITH_LEN("panic: pthread_key_create failed\n"))); \
 	    exit(1);						\
 	}							\
     } STMT_END
@@ -370,47 +378,47 @@
 #endif /* USE_ITHREADS */
 
 #ifndef MUTEX_LOCK
-#  define MUTEX_LOCK(m)
+#  define MUTEX_LOCK(m)           NOOP
 #endif
 
 #ifndef MUTEX_UNLOCK
-#  define MUTEX_UNLOCK(m)
+#  define MUTEX_UNLOCK(m)         NOOP
 #endif
 
 #ifndef MUTEX_INIT
-#  define MUTEX_INIT(m)
+#  define MUTEX_INIT(m)           NOOP
 #endif
 
 #ifndef MUTEX_DESTROY
-#  define MUTEX_DESTROY(m)
+#  define MUTEX_DESTROY(m)        NOOP
 #endif
 
 #ifndef COND_INIT
-#  define COND_INIT(c)
+#  define COND_INIT(c)            NOOP
 #endif
 
 #ifndef COND_SIGNAL
-#  define COND_SIGNAL(c)
+#  define COND_SIGNAL(c)          NOOP
 #endif
 
 #ifndef COND_BROADCAST
-#  define COND_BROADCAST(c)
+#  define COND_BROADCAST(c)       NOOP
 #endif
 
 #ifndef COND_WAIT
-#  define COND_WAIT(c, m)
+#  define COND_WAIT(c, m)         NOOP
 #endif
 
 #ifndef COND_DESTROY
-#  define COND_DESTROY(c)
+#  define COND_DESTROY(c)         NOOP
 #endif
 
 #ifndef LOCK_DOLLARZERO_MUTEX
-#  define LOCK_DOLLARZERO_MUTEX
+#  define LOCK_DOLLARZERO_MUTEX   NOOP
 #endif
 
 #ifndef UNLOCK_DOLLARZERO_MUTEX
-#  define UNLOCK_DOLLARZERO_MUTEX
+#  define UNLOCK_DOLLARZERO_MUTEX NOOP
 #endif
 
 /* THR, SET_THR, and dTHR are there for compatibility with old versions */
@@ -431,11 +439,5 @@
 #endif
 
 /*
- * Local variables:
- * c-indentation-style: bsd
- * c-basic-offset: 4
- * indent-tabs-mode: nil
- * End:
- *
  * ex: set ts=8 sts=4 sw=4 et:
  */
